@@ -74,31 +74,6 @@ class ExplicitComponent(Component):
         if self.matrix_free:
             self._check_matfree_deprecation()
 
-    def _get_partials_varlists(self, use_resname=False):
-        """
-        Get lists of 'of' and 'wrt' variables that form the partial jacobian.
-
-        Parameters
-        ----------
-        use_resname : bool
-            Ignored for explicit components.
-
-        Returns
-        -------
-        tuple(list, list)
-            'of' and 'wrt' variable lists.
-        """
-        of = list(self._var_rel_names['output'])
-        wrt = list(self._var_rel_names['input'])
-
-        # filter out any discrete inputs or outputs
-        if self._discrete_outputs:
-            of = [n for n in of if n not in self._discrete_outputs]
-        if self._discrete_inputs:
-            wrt = [n for n in wrt if n not in self._discrete_inputs]
-
-        return of, wrt
-
     def _jac_wrt_iter(self, wrt_matches=None):
         """
         Iterate over (name, start, end, vec, slice, dist_sizes) for each column var in the jacobian.
@@ -137,6 +112,15 @@ class ExplicitComponent(Component):
                 yield wrt, start, end, vec, _full_slice, dist_sizes
                 start = end
 
+    def _setup_residuals(self):
+        """
+        Prevent the user from implementing setup_residuals for explicit components.
+        """
+        if overrides_method('setup_residuals', self, ExplicitComponent):
+            raise RuntimeError(f'{self.msginfo}: Class overrides setup_residuals but '
+                               'is an ExplicitComponent. setup_residuals may only be '
+                               'overridden by ImplicitComponents.')
+
     def _setup_partials(self):
         """
         Call setup_partials in components.
