@@ -7,13 +7,16 @@ import itertools
 
 from collections import OrderedDict
 
+from fnmatch import fnmatchcase
+
 import numpy as np
 
 from openmdao.core.constants import _DEFAULT_OUT_STREAM
+from openmdao.core.system import allowed_meta_names
 from openmdao.recorders.sqlite_recorder import blob_to_array
 from openmdao.utils.record_util import deserialize, get_source_system
-from openmdao.utils.variable_table import write_var_table
-from openmdao.utils.general_utils import make_set, match_prom_or_abs
+from openmdao.utils.variable_table import write_var_table, NA
+from openmdao.utils.general_utils import match_prom_or_abs
 from openmdao.utils.units import unit_conversion, simplify_unit
 from openmdao.recorders.sqlite_recorder import format_version as current_version
 
@@ -392,15 +395,363 @@ class Case(object):
         """
         return self._get_variables_of_type('response', scaled, use_indices)
 
+    def get_io_metadata(self, iotypes=('input', 'output'), metadata_keys=None,
+                        includes=None, excludes=None, is_indep_var=None, is_design_var=None,
+                        tags=None):
+        """
+        Retrieve metadata for a filtered list of variables.
+
+        Parameters
+        ----------
+        iotypes : str or iter of str
+            Will contain either 'input', 'output', or both.  Defaults to both.
+        metadata_keys : iter of str or None
+            Names of metadata entries to be retrieved or None, meaning retrieve all
+            available 'allprocs' metadata.  If 'val' or 'src_indices' are required,
+            their keys must be provided explicitly since they are not found in the 'allprocs'
+            metadata and must be retrieved from local metadata located in each process.
+        includes : str, iter of str or None
+            Collection of glob patterns for pathnames of variables to include. Default is None,
+            which includes all variables.
+        excludes : str, iter of str or None
+            Collection of glob patterns for pathnames of variables to exclude. Default is None.
+        is_indep_var : bool or None
+            If None (the default), do no additional filtering of the inputs.
+            If True, list only inputs connected to an output tagged `openmdao:indep_var`.
+            If False, list only inputs _not_ connected to outputs tagged `openmdao:indep_var`.
+        is_design_var : bool or None
+            If None (the default), do no additional filtering of the inputs.
+            If True, list only outputs and connected inputs that are driver design variables.
+            If False, list only outputs and connected inputs that are _not_ driver design variables.
+        tags : str or iter of strs
+            User defined tags that can be used to filter what gets listed. Only inputs with the
+            given tags will be listed.
+            Default is None, which means there will be no filtering based on tags.
+
+        Returns
+        -------
+        dict
+            A dict of metadata keyed on name, where the metadata is a dict containing
+            entries based on the value of the metadata_keys arg.  Every metadata dict will
+            always contain two entries, 'prom_name' and 'discrete', to indicate a given
+            variable's promoted name and whether or not it is discrete.
+        """
+        if isinstance(iotypes, str):
+            iotypes = (iotypes,)
+        if isinstance(includes, str):
+            includes = (includes,)
+        if isinstance(excludes, str):
+            excludes = (excludes,)
+        if isinstance(tags, str):
+            tags = {tags}
+
+        if metadata_keys is not None:
+            keyset = set(metadata_keys)
+            diff = keyset - allowed_meta_names
+            if diff:
+                raise RuntimeError(f"Case: {sorted(diff)} are not valid metadata entry names.")
+
+        abs2meta = self._abs2meta
+        abs2prom = self._abs2prom
+
+        result = {}
+
+        if is_design_var is not None:
+            des_vars = self.get_design_vars()
+            auto_ivc_map = self._auto_ivc_map
+
+        for iotype in iotypes:
+            data = getattr(self, f'{iotype}s')
+
+            if data is None:
+                # data not recorded for this i/o type
+                continue
+
+            for abs_name in data.absolute_names():
+                prom = abs2prom[iotype][abs_name]
+
+                if not match_prom_or_abs(abs_name, prom, includes, excludes):
+                    continue
+
+                meta = abs2meta[abs_name] if abs_name in abs2meta else None
+
+                if meta is None:
+                    continue
+
+                if metadata_keys is None:
+                    ret_meta = dict(meta)
+                else:
+                    ret_meta = {}
+                    for key in keyset:
+                        try:
+                            ret_meta[key] = tuple(meta[key]) if 'shape' in key else meta[key]
+                        except KeyError:
+                            ret_meta[key] = NA
+
+                # handle is_indep_var
+                if is_indep_var is not None:
+                    if iotype == 'output':
+                        out_meta = meta
+                    else:
+                        src_name = self._conns[abs_name]
+                        out_meta = abs2meta[src_name]
+
+                    src_tags = out_meta['tags'] if 'tags' in out_meta else {}
+                    if is_indep_var:
+                        if 'openmdao:indep_var' not in src_tags:
+                            continue
+                    elif 'openmdao:indep_var' in src_tags:
+                        continue
+
+                # handle is_design_var
+                if is_design_var is not None:
+                    if iotype == 'output':
+                        out_name = abs_name
+                    else:
+                        # input, get connected output
+                        src_name = self._conns[abs_name]
+                        out_name = abs2prom['output'][src_name]
+
+                    if out_name.startswith('_auto_ivc.'):
+                        out_name = auto_ivc_map[out_name]
+
+                    if is_design_var:
+                        if out_name not in des_vars:
+                            continue
+                    elif out_name in des_vars:
+                        continue
+
+                # handle tags
+                if tags:
+                    meta_tags = ret_meta.get('tags', {})
+                    match_tag = False
+                    for tag in tags:
+                        for meta_tag in meta_tags:
+                            if fnmatchcase(meta_tag, tag):
+                                match_tag = True
+                                break
+                    if not match_tag:
+                        continue
+
+                ret_meta['io'] = iotype
+
+                ret_meta['discrete'] = 'discrete' in abs2meta[abs_name]
+                ret_meta['prom_name'] = prom
+
+                if iotype == 'output':
+                    ret_meta['explicit'] = True if meta.get('explicit') else False
+
+                result[abs_name] = ret_meta
+
+        return result
+
+    def list_vars(self,
+                  val=True,
+                  prom_name=True,
+                  residuals=False,
+                  residuals_tol=None,
+                  units=False,
+                  shape=False,
+                  bounds=False,
+                  scaling=False,
+                  desc=False,
+                  print_arrays=False,
+                  tags=None,
+                  print_tags=False,
+                  includes=None,
+                  excludes=None,
+                  is_indep_var=None,
+                  is_design_var=None,
+                  list_autoivcs=False,
+                  out_stream=_DEFAULT_OUT_STREAM,
+                  print_min=False,
+                  print_max=False,
+                  return_format='list'):
+        """
+        Write a list of inputs and outputs sorted by component in execution order.
+
+        Parameters
+        ----------
+        val : bool, optional
+            When True, display output values. Default is True.
+        prom_name : bool, optional
+            When True, display the promoted name of the variable.
+            Default is True.
+        residuals : bool, optional
+            When True, display residual values. Default is False.
+        residuals_tol : float, optional
+            If set, limits the output of list_outputs to only variables where
+            the norm of the resids array is greater than the given 'residuals_tol'.
+            Default is None.
+        units : bool, optional
+            When True, display units. Default is False.
+        shape : bool, optional
+            When True, display/return the shape of the value. Default is False.
+        bounds : bool, optional
+            When True, display/return bounds (lower and upper). Default is False.
+        scaling : bool, optional
+            When True, display/return scaling (ref, ref0, and res_ref). Default is False.
+        desc : bool, optional
+            When True, display/return description. Default is False.
+        print_arrays : bool, optional
+            When False, in the columnar display, just display norm of any ndarrays with size > 1.
+            The norm is surrounded by vertical bars to indicate that it is a norm.
+            When True, also display full values of the ndarray below the row. Format  is affected
+            by the values set with numpy.set_printoptions
+            Default is False.
+        tags : str or list of strs
+            User defined tags that can be used to filter what gets listed. Only outputs with the
+            given tags will be listed.
+            Default is None, which means there will be no filtering based on tags.
+        print_tags : bool
+            When true, display tags in the columnar display.
+        includes : None, str, or iter of str
+            Collection of glob patterns for pathnames of variables to include. Default is None,
+            which includes all output variables.
+        excludes : None, str, or iter of str
+            Collection of glob patterns for pathnames of variables to exclude. Default is None.
+        is_indep_var : bool or None
+            If None (the default), do no additional filtering of the inputs.
+            If True, list only outputs tagged `openmdao:indep_var`.
+            If False, list only outputs that are _not_ tagged `openmdao:indep_var`.
+        is_design_var : bool or None
+            If None (the default), do no additional filtering of the inputs.
+            If True, list only inputs connected to outputs that are driver design variables.
+            If False, list only inputs _not_ connected to outputs that are driver design variables.
+        list_autoivcs : bool
+            If True, include auto_ivc outputs in the listing.  Defaults to False.
+        out_stream : file-like
+            Where to send human readable output. Default is sys.stdout.
+            Set to None to suppress.
+        print_min : bool
+            When true, if the output value is an array, print its smallest value.
+        print_max : bool
+            When true, if the output value is an array, print its largest value.
+        return_format : str
+            Indicates the desired format of the return value. Can have value of 'list' or 'dict'.
+            If 'list', the return value is a list of (name, metadata) tuples.
+            if 'dict', the return value is a dictionary mapping {name: metadata}.
+
+        Returns
+        -------
+        list of (name, metadata) or dict of {name: metadata}
+            List or dict of output names and other optional information about those outputs.
+        """
+        if return_format not in ('list', 'dict'):
+            badarg = f"'{return_format}'" if isinstance(return_format, str) else f"{return_format}"
+            raise ValueError(f"Invalid value ({badarg}) for return_format, "
+                             "must be a string value of 'list' or 'dict'")
+
+        keynames = ['val', 'units', 'shape', 'desc', 'tags']
+        keyflags = [val, units, shape, desc, tags or print_tags]
+        keys = [name for i, name in enumerate(keynames) if keyflags[i]]
+
+        if bounds:
+            keys.extend(('lower', 'upper'))
+        if scaling:
+            keys.extend(('ref', 'ref0', 'res_ref'))
+
+        outputs = self.get_io_metadata('output', keys, includes, excludes,
+                                       is_indep_var, is_design_var, tags)
+
+        # filter auto_ivcs if requested
+        if outputs and not list_autoivcs:
+            outputs = {n: m for n, m in outputs.items() if not n.startswith('_auto_ivc.')}
+
+        # get output values & resids
+        if outputs and (val or residuals or residuals_tol):
+            to_remove = []
+            print_options = np.get_printoptions()
+            np_precision = print_options['precision']
+
+            for name, meta in outputs.items():
+                if val:
+                    # we want value from the case, not from the metadata
+                    meta['val'] = self.outputs[name]
+
+                    if isinstance(meta['val'], np.ndarray):
+                        if print_min:
+                            meta['min'] = np.round(np.min(meta['val']), np_precision)
+
+                        if print_max:
+                            meta['max'] = np.round(np.max(meta['val']), np_precision)
+
+                if residuals or residuals_tol:
+                    resids = self.residuals[name]
+                    if residuals_tol and np.linalg.norm(resids) < residuals_tol:
+                        to_remove.append(name)
+                    elif residuals:
+                        meta['resids'] = resids
+
+            # remove any outputs that don't pass the residuals_tol filter
+            for name in to_remove:
+                del outputs[name]
+
+        if self.inputs is not None:
+            inputs = self.get_io_metadata('input', keys, includes, excludes,
+                                          is_indep_var, is_design_var, tags)
+        else:
+            inputs = {}
+
+        # get input values
+        if inputs and val is not None:
+            print_options = np.get_printoptions()
+            np_precision = print_options['precision']
+
+            for name, meta in inputs.items():
+                # we want value from the case, not from the metadata
+                meta['val'] = self.inputs[name]
+
+                if isinstance(meta['val'], np.ndarray):
+                    if print_min:
+                        meta['min'] = np.round(np.min(meta['val']), np_precision)
+
+                    if print_max:
+                        meta['max'] = np.round(np.max(meta['val']), np_precision)
+
+        # combine inputs and outputs create return value
+        var_dict = inputs
+        var_dict.update(outputs)
+
+        # remove metadata we don't want to show/return
+        to_remove = ['discrete', 'explicit']
+        if not prom_name:
+            to_remove.append('prom_name')
+        if not print_tags:
+            to_remove.append('tags')
+        for meta in var_dict.values():
+            for key in to_remove:
+                try:
+                    del meta[key]
+                except KeyError:
+                    pass
+
+        if var_dict:
+            if 'execution_order' in self._var_info:
+                var_list = []
+                for var_name in self._var_info['execution_order']:
+                    if var_name in var_dict:
+                        var_list.append(var_name)
+            else:
+                # don't have execution order, just sort for determinism
+                var_list = sorted(var_dict.keys())
+
+            write_var_table('', var_list, 'all', var_dict,
+                            True, print_arrays, out_stream)
+
+        return var_dict if return_format == 'dict' else list(var_dict.items())
+
     def list_inputs(self,
                     val=True,
                     prom_name=True,
                     units=False,
                     shape=False,
+                    global_shape=False,
                     desc=False,
                     hierarchical=True,
                     print_arrays=False,
                     tags=None,
+                    print_tags=False,
                     includes=None,
                     excludes=None,
                     is_indep_var=None,
@@ -423,6 +774,8 @@ class Case(object):
             When True, display/return units. Default is False.
         shape : bool, optional
             When True, display/return the shape of the value. Default is False.
+        global_shape : bool, optional
+            When True, display/return the global shape of the value. Default is False.
         desc : bool, optional
             When True, display/return description. Default is False.
         hierarchical : bool, optional
@@ -437,6 +790,8 @@ class Case(object):
             User defined tags that can be used to filter what gets listed. Only inputs with the
             given tags will be listed.
             Default is None, which means there will be no filtering based on tags.
+        print_tags : bool
+            When true, display tags in the columnar display.
         includes : str, iter of str, or None
             Glob patterns for pathnames to include in the check. Default is None, which
             includes all.
@@ -473,83 +828,27 @@ class Case(object):
             raise ValueError(f"Invalid value ({badarg}) for return_format, "
                              "must be a string value of 'list' or 'dict'")
 
-        abs2meta = self._abs2meta
-        inputs = []
+        if not self.inputs:
+            return {} if return_format == 'dict' else []
 
-        # string to display when an attribute is not available (e.g. for a discrete)
-        NA = 'Unavailable'
+        keynames = ['val', 'units', 'shape', 'global_shape', 'desc', 'tags']
+        keyvals = [val, units, shape, global_shape, desc, tags or print_tags]
+        keys = [n for i, n in enumerate(keynames) if keyvals[i]]
 
-        if isinstance(includes, str):
-            includes = [includes, ]
+        inputs = self.get_io_metadata('input', keys, includes, excludes,
+                                      is_indep_var, is_design_var, tags)
 
-        if isinstance(excludes, str):
-            excludes = [excludes, ]
-
-        if self.inputs is not None:
+        if inputs and val:
             print_options = np.get_printoptions()
             np_precision = print_options['precision']
 
-            if is_design_var is not None:
-                des_vars = self._get_variables_of_type('desvar')
-
-            for var_name in self.inputs.absolute_names():
-                meta = abs2meta[var_name]
-
-                # Filter based on tags
-                if tags and not (make_set(tags) & make_set(meta['tags'])):
-                    continue
-
-                var_name_prom = self._abs2prom['input'][var_name]
-
-                if not match_prom_or_abs(var_name, var_name_prom, includes, excludes):
-                    continue
-
-                # handle is_indep_var
-                if is_indep_var is not None:
-                    src_name = self._conns[var_name]
-                    src_name_prom = self._abs2prom['output'][src_name]
-                    src_meta = abs2meta[src_name]
-                    if is_indep_var is True and 'openmdao:indep_var' not in src_meta['tags']:
-                        continue
-                    elif is_indep_var is False and 'openmdao:indep_var' in src_meta['tags']:
-                        continue
-
-                # handle is_design_var
-                if is_design_var is not None:
-                    src_name = self._conns[var_name]
-                    src_name_prom = self._abs2prom['output'][src_name]
-                    if src_name_prom.startswith('_auto_ivc.'):
-                        src_name_prom = self._auto_ivc_map[src_name_prom]
-                    if is_design_var is True and src_name_prom not in des_vars:
-                        continue
-                    elif is_design_var is False and src_name_prom in des_vars:
-                        continue
-
-                var_val = self.inputs[var_name]
-
-                var_meta = {}
-                if val:
-                    var_meta['val'] = var_val
-                    if isinstance(var_val, np.ndarray):
-                        if print_min:
-                            var_meta['min'] = np.round(np.min(var_val), np_precision)
-
-                        if print_max:
-                            var_meta['max'] = np.round(np.max(var_val), np_precision)
-
-                if prom_name:
-                    var_meta['prom_name'] = var_name_prom
-                if units:
-                    var_meta['units'] = meta.get('units', NA)
-                if shape:
-                    try:
-                        var_meta['shape'] = var_val.shape
-                    except AttributeError:
-                        var_meta['shape'] = NA
-                if desc:
-                    var_meta['desc'] = meta['desc']
-
-                inputs.append((var_name, var_meta))
+            for name, meta in inputs.items():
+                meta['val'] = var_val = self.inputs[name]
+                if isinstance(var_val, np.ndarray):
+                    if print_min:
+                        meta['min'] = np.round(np.min(var_val), np_precision)
+                    if print_max:
+                        meta['max'] = np.round(np.max(var_val), np_precision)
 
         if out_stream:
             if self.inputs:
@@ -559,10 +858,20 @@ class Case(object):
                 ostream.write('WARNING: Inputs not recorded. Make sure your recording ' +
                               'settings have record_inputs set to True\n')
 
-        if return_format == 'dict':
-            return dict(inputs)
-        else:
-            return inputs
+        # remove metadata we don't want to show/return
+        to_remove = ['discrete']
+        if not prom_name:
+            to_remove.append('prom_name')
+        if not print_tags:
+            to_remove.append('tags')
+        for meta in inputs.values():
+            for key in to_remove:
+                try:
+                    del meta[key]
+                except KeyError:
+                    pass
+
+        return inputs if return_format == 'dict' else list(inputs.items())
 
     def list_outputs(self,
                      explicit=True, implicit=True,
@@ -572,12 +881,14 @@ class Case(object):
                      residuals_tol=None,
                      units=False,
                      shape=False,
+                     global_shape=False,
                      bounds=False,
                      scaling=False,
                      desc=False,
                      hierarchical=True,
                      print_arrays=False,
                      tags=None,
+                     print_tags=False,
                      includes=None,
                      excludes=None,
                      is_indep_var=None,
@@ -611,6 +922,8 @@ class Case(object):
             When True, display/return units. Default is False.
         shape : bool, optional
             When True, display/return the shape of the value. Default is False.
+        global_shape : bool, optional
+            When True, display/return the global shape of the value. Default is False.
         bounds : bool, optional
             When True, display/return bounds (lower and upper). Default is False.
         scaling : bool, optional
@@ -629,6 +942,8 @@ class Case(object):
             User defined tags that can be used to filter what gets listed. Only outputs with the
             given tags will be listed.
             Default is None, which means there will be no filtering based on tags.
+        print_tags : bool
+            When true, display tags in the columnar display.
         includes : str, iter of str, or None
             Glob patterns for pathnames to include in the check. Default is None, which
             includes all.
@@ -667,100 +982,73 @@ class Case(object):
             raise ValueError(f"Invalid value ({badarg}) for return_format, "
                              "must be a string value of 'list' or 'dict'")
 
-        abs2meta = self._abs2meta
-        expl_outputs = []
-        impl_outputs = []
+        if not self.outputs:
+            return {} if return_format == 'dict' else []
 
-        # string to display when an attribute is not available (e.g. for a discrete)
-        NA = 'Unavailable'
+        keynames = ['val', 'units', 'shape', 'global_shape', 'desc', 'tags']
+        keyvals = [val, units, shape, global_shape, desc, tags or print_tags]
+        keys = [n for i, n in enumerate(keynames) if keyvals[i]]
 
-        if isinstance(includes, str):
-            includes = [includes, ]
+        if bounds:
+            keys.extend(('lower', 'upper'))
+        if scaling:
+            keys.extend(('ref', 'ref0', 'res_ref'))
 
-        if isinstance(excludes, str):
-            excludes = [excludes, ]
+        outputs = self.get_io_metadata('output', keys, includes, excludes,
+                                       is_indep_var, is_design_var, tags)
 
-        print_options = np.get_printoptions()
-        np_precision = print_options['precision']
+        # filter auto_ivcs if requested
+        if outputs and not list_autoivcs:
+            outputs = {n: m for n, m in outputs.items() if not n.startswith('_auto_ivc.')}
 
-        if is_design_var is not None:
-            des_vars = self._get_variables_of_type('desvar')
+        # get output values & resids
+        if outputs and (val or residuals or residuals_tol):
+            to_remove = []
+            print_options = np.get_printoptions()
+            np_precision = print_options['precision']
 
-        for var_name in self.outputs.absolute_names():
-            if not list_autoivcs and var_name.startswith('_auto_ivc.'):
-                continue
+            for name, meta in outputs.items():
+                if val:
+                    # we want value from the case, not from the metadata
+                    meta['val'] = self.outputs[name]
 
-            meta = abs2meta[var_name]
+                    if isinstance(meta['val'], np.ndarray):
+                        if print_min:
+                            meta['min'] = np.round(np.min(meta['val']), np_precision)
 
-            # Filter based on tags
-            if tags and not (make_set(tags) & make_set(meta['tags'])):
-                continue
+                        if print_max:
+                            meta['max'] = np.round(np.max(meta['val']), np_precision)
 
-            var_name_prom = self._abs2prom['output'][var_name]
+                if residuals or residuals_tol:
+                    try:
+                        resids = self.residuals[name]
+                        if residuals_tol and np.linalg.norm(resids) < residuals_tol:
+                            to_remove.append(name)
+                        elif residuals:
+                            meta['resids'] = resids
+                    except KeyError:
+                        if residuals:
+                            meta['resids'] = 'Not Recorded'
 
-            if not match_prom_or_abs(var_name, var_name_prom, includes, excludes):
-                continue
+            # remove any outputs that don't pass the residuals_tol filter
+            for name in to_remove:
+                del outputs[name]
 
-            # handle is_indep_var
-            if is_indep_var is not None:
-                if is_indep_var is True and 'openmdao:indep_var' not in meta['tags']:
-                    continue
-                elif is_indep_var is False and 'openmdao:indep_var' in meta['tags']:
-                    continue
+        expl_outputs = {n: m for n, m in outputs.items() if m['explicit']}
+        impl_outputs = {n: m for n, m in outputs.items() if not m['explicit']}
 
-            # handle is_design_var
-            if is_design_var is not None:
-                var_name_prom = self._abs2prom['output'][var_name]
-                if var_name_prom.startswith('_auto_ivc.'):
-                    var_name_prom = self._auto_ivc_map[var_name_prom]
-                if is_design_var is True and var_name_prom not in des_vars:
-                    continue
-                elif is_design_var is False and var_name_prom in des_vars:
-                    continue
-
-            # check if residuals were recorded, skip if within specifed tolerance
-            if residuals and self.residuals and var_name in self.residuals.absolute_names():
-                resids = self.residuals[var_name]
-                if residuals_tol and np.linalg.norm(resids) < residuals_tol:
-                    continue
-            else:
-                resids = 'Not Recorded'
-
-            var_val = self.outputs[var_name]
-
-            var_meta = {}
-            if val:
-                var_meta['val'] = var_val
-                if isinstance(var_val, np.ndarray):
-                    if print_min:
-                        var_meta['min'] = np.round(np.min(var_val), np_precision)
-
-                    if print_max:
-                        var_meta['max'] = np.round(np.max(var_val), np_precision)
-            if prom_name:
-                var_meta['prom_name'] = var_name_prom
-            if residuals:
-                var_meta['resids'] = resids
-            if units:
-                var_meta['units'] = meta.get('units', NA)
-            if shape:
+        # remove metadata we don't want to show/return
+        to_remove = ['discrete', 'explicit']
+        if not prom_name:
+            to_remove.append('prom_name')
+        if not print_tags:
+            to_remove.append('tags')
+        for meta in itertools.chain(outputs.values(), expl_outputs.values(), impl_outputs.values()):
+            for key in to_remove:
                 try:
-                    var_meta['shape'] = var_val.shape
-                except AttributeError:
-                    var_meta['shape'] = NA
-            if bounds:
-                var_meta['lower'] = meta.get('lower', NA)
-                var_meta['upper'] = meta.get('upper', NA)
-            if scaling:
-                var_meta['ref'] = meta.get('ref', NA)
-                var_meta['ref0'] = meta.get('ref0', NA)
-                var_meta['res_ref'] = meta.get('res_ref', NA)
-            if desc:
-                var_meta['desc'] = meta['desc']
-            if meta['explicit']:
-                expl_outputs.append((var_name, var_meta))
-            else:
-                impl_outputs.append((var_name, var_meta))
+                    del meta[key]
+                except KeyError:
+                    pass
 
         if out_stream:
             if not self.outputs:
@@ -773,7 +1061,7 @@ class Case(object):
                 self._write_table('implicit', impl_outputs, hierarchical, print_arrays, out_stream)
 
         if explicit and implicit:
-            outputs = expl_outputs + impl_outputs
+            pass
         elif explicit:
             outputs = expl_outputs
         elif implicit:
@@ -781,10 +1069,7 @@ class Case(object):
         else:
             raise RuntimeError('You have excluded both Explicit and Implicit components.')
 
-        if return_format == 'dict':
-            return dict(outputs)
-        else:
-            return outputs
+        return outputs if return_format == 'dict' else list(outputs.items())
 
     def _write_table(self, var_type, var_data, hierarchical, print_arrays, out_stream):
         """
@@ -794,8 +1079,8 @@ class Case(object):
         ----------
         var_type : 'input', 'explicit' or 'implicit'
             Indicates type of variables, input or explicit/implicit output.
-        var_data : list
-            List of (name, dict of vals and metadata) tuples.
+        var_data : dict or list
+            Dict of {name: metadata} or list of (name, metadata).
         hierarchical : bool
             When True, human readable output shows variables in hierarchical format.
         print_arrays : bool
@@ -812,9 +1097,12 @@ class Case(object):
             return
 
         # Make a dict of variables. Makes it easier to work with in this method
-        var_dict = OrderedDict()
-        for name, vals in var_data:
-            var_dict[name] = vals
+        if isinstance(var_data, dict):
+            var_dict = var_data
+        else:
+            var_dict = OrderedDict()
+            for name, meta in var_data:
+                var_dict[name] = meta
 
         # determine pathname of the system
         if self.source in ('root', 'driver', 'problem', 'root.nonlinear_solver'):
@@ -834,10 +1122,9 @@ class Case(object):
             # don't have execution order, just sort for determinism
             var_list = sorted(var_dict.keys())
 
-        top_name = pathname if pathname else 'model'
         write_var_table(pathname, var_list, var_type, var_dict,
-                        hierarchical=hierarchical, top_name=top_name,
-                        print_arrays=print_arrays, out_stream=out_stream)
+                        hierarchical=hierarchical, print_arrays=print_arrays,
+                        out_stream=out_stream)
 
     def _get_variables_of_type(self, var_type, scaled=False, use_indices=False):
         """
